"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http = require("tns-core-modules/http");
var Common = (function () {
    function Common(options) {
        this.constants = {
            sdkVersion: "nativescript-dec-sdk-1.0.2",
            endpoints: {
                apiServerUrl: "https://api.dec.sitefinity.com"
            },
            headers: {
                authorization: "Authorization",
                datacenterkey: "x-dataintelligence-datacenterkey",
                subject: "x-dataintelligence-subject",
                ids: "x-dataintelligence-ids",
                datasource: "x-dataintelligence-datasource",
                contacts: "x-dataintelligence-contacts",
                sdkVersion: "x-dataintelligence-sdk-version"
            }
        };
        if (!options.apiKey)
            throw new Error('You must provide the ApiKey for the Data Center.');
        if (!options.source)
            throw new Error('You must provide the source name of the client.');
        this.apiKey = options.apiKey;
        this.source = options.source;
        this.authToken = options.authToken;
        this.apiServerUrl = options.apiServerUrl || this.constants.endpoints.apiServerUrl;
        this.interactions = [];
    }
    Common.prototype.writeInteraction = function (interaction) {
        if (!interaction)
            throw new Error('You must provide "interaction" argument when writing an interaction.');
        if (!interaction.S)
            throw new Error('The interaction must contain SubjectKey.');
        this.interactions.push(interaction);
    };
    Common.prototype.writeSubjectMetadata = function (subjectKey, metadata) {
        if (!subjectKey)
            throw new Error('You must provide "subjectKey" argument when writing subject metadata.');
        if (!metadata)
            throw new Error('You must provide "metadata" argument when writing subject metadata.');
        this.writeInteraction({
            S: subjectKey,
            SM: metadata
        });
    };
    Common.prototype.writeObjectMetadata = function (subjectKey, metadata) {
        if (!subjectKey)
            throw new Error('You must provide "subjectKey" argument when writing subject metadata.');
        if (!metadata)
            throw new Error('You must provide "metadata" argument when writing subject metadata.');
        this.writeInteraction({
            S: subjectKey,
            OM: metadata
        });
    };
    Common.prototype.addMapping = function (subjectKey, secondSubjectKey, secondDataSource) {
        this.writeInteraction({
            S: subjectKey,
            MappedTo: [{
                    "S": secondSubjectKey,
                    "DS": secondDataSource
                }]
        });
    };
    Common.prototype.buildPersonalizationReportInteraction = function (personalizationReportParams) {
        var interaction = {
            S: personalizationReportParams.subjectKey,
            P: 'Visit',
            O: personalizationReportParams.canonicalUrl + '/',
            OM: {
                Id: personalizationReportParams.pageGuid,
                PageId: personalizationReportParams.pageGuid,
                Language: personalizationReportParams.language,
                ContentType: 'Page',
                CanonicalTitle: personalizationReportParams.canonicalTitle,
                CanonicalUrl: personalizationReportParams.canonicalUrl,
                Personalization: [{
                        Type: 'Page',
                        Segment: personalizationReportParams.segment
                    }]
            }
        };
        return interaction;
    };
    Common.prototype.flushData = function () {
        var _this = this;
        var requestOptions = this.getInteractionsRequestOptions();
        this.makeRequest(requestOptions)
            .then(function (response) {
            _this.interactions = [];
        })
            .catch(function (response) {
            throw new Error("The sending of Interactions to DEC server failed!");
        });
    };
    Common.prototype.isInCampaigns = function (campaignIds, subjectKey) {
        if (!campaignIds)
            throw new Error('You must provide "campaignIds" argument when using the "isInCampaigns" method.');
        if (!subjectKey)
            throw new Error('You must provide "subjectKey" argument when using the "isInCampaigns" method.');
        var headers = {};
        headers[this.constants.headers.subject] = subjectKey;
        headers[this.constants.headers.ids] = campaignIds;
        headers[this.constants.headers.datasource] = this.source;
        return this.makePersonalizationRequest('/analytics/v1/campaigns/isin', headers);
    };
    Common.prototype.isInLeads = function (scoringIds, subjectKey) {
        if (!scoringIds)
            throw new Error('You must provide "scoringIds" argument when using the "isInLeads" method.');
        if (!subjectKey)
            throw new Error('You must provide "subjectKey" argument when using the "isInLeads" method.');
        var headers = {};
        headers[this.constants.headers.subject] = subjectKey;
        headers[this.constants.headers.ids] = scoringIds;
        headers[this.constants.headers.datasource] = this.source;
        return this.makePersonalizationRequest('/analytics/v2/scorings/leads/in', headers);
    };
    Common.prototype.isInPersonas = function (scoringIds, subjectKey) {
        if (!scoringIds)
            throw new Error('You must provide "scoringIds" argument when using the "isInPersonas" method.');
        if (!subjectKey)
            throw new Error('You must provide "subjectKey" argument when using the "isInPersonas" method.');
        var headers = {};
        headers[this.constants.headers.subject] = subjectKey;
        headers[this.constants.headers.ids] = scoringIds;
        headers[this.constants.headers.datasource] = this.source;
        return this.makePersonalizationRequest('/analytics/v1/scorings/personas/in', headers);
    };
    Common.prototype.getEndpointUrl = function () {
        return this.apiServerUrl + '/collect/v2/data-centers/' + this.apiKey + '/datasources/' + this.source + '/interactions';
    };
    Common.prototype.getInteractionsRequestOptions = function (headers) {
        var url = this.getEndpointUrl();
        var requestOptions = this.getRequestOptions(this.interactions, 'POST', url, headers);
        return requestOptions;
    };
    Common.prototype.getRequestOptions = function (data, httpMethod, url, headers) {
        var requestOptions = {
            method: httpMethod,
            url: url,
            contentType: 'application/json',
            headers: headers || {},
            content: data ? JSON.stringify(data) : null
        };
        requestOptions.headers[this.constants.headers.sdkVersion] = this.constants.sdkVersion;
        return requestOptions;
    };
    Common.prototype.makePersonalizationRequest = function (endpoint, headers) {
        if (!this.authToken)
            throw new Error('You must provide "authToken" when using the Personalization Client.');
        var url = this.apiServerUrl + endpoint;
        var requestOptions = this.getRequestOptions(null, 'GET', url, headers);
        requestOptions.headers[this.constants.headers.authorization] = this.authToken;
        requestOptions.headers[this.constants.headers.datacenterkey] = this.apiKey;
        return this.makeRequest(requestOptions);
    };
    Common.prototype.makeRequest = function (requestOptions) {
        if (!requestOptions)
            throw new Error("options is required.");
        if (!requestOptions.method)
            throw new Error("options.method is required.");
        if (!requestOptions.url)
            throw new Error("options.url is required.");
        return new Promise(function (resolve, reject) {
            if (requestOptions.contentType) {
                requestOptions.headers["Content-Type"] = requestOptions.contentType;
            }
            http.request(requestOptions).then(function (response) {
                if ((response.statusCode >= 200 && response.statusCode < 300) || response.statusCode === 304) {
                    resolve(response.content);
                }
                else {
                    reject(response.content);
                }
            }, function (error) {
                reject(error);
            });
        });
    };
    return Common;
}());
exports.Common = Common;
//# sourceMappingURL=dec-sdk.common.js.map